#include <Wire.h>
#include <Ultrasonic.h>
#define PWM13       OCR4A
#define TRIGGER_PIN_1 7
#define ECHO_PIN_1 8
#define TRIGGER_PIN_2 9
#define ECHO_PIN_2 10

Ultrasonic sensor1(TRIGGER_PIN_1, ECHO_PIN_1);
Ultrasonic sensor2(TRIGGER_PIN_2, ECHO_PIN_2);
//  SETUP I2C
byte DRV = 0x5A;    //DRV2605 slave address
byte ModeReg = 0x01;
byte TCA = 0x70 ;
const int origin = 10;
const int limit = 50;
int freqmin = 50;
int freqmax = 300;
float distance1;
float distance2;
float slope1;
float freq1;
float slope2;
float freq2;
int ports[2] = {1,2};

void setup() {
  Wire.begin();
  //while(!Serial){}; // Sequence will not run until we activate the serial port (or monitor)
  Serial.begin(9600); 
  pwm13configure();    //sets up PWM signal
  delay(2);
  initializeDRV2605();  //initializes DRV2605

}
  
void loop() {

  // Read the distance from the first sensor
  distance1 = sensor1.read();
  // Read the distance from the second sensor
  distance2 = sensor2.read();
  // Verhoog de frequentie als object dichterbij komt
  slope1 = (freqmax-freqmin)/(origin-limit);
  freq1 = (slope1*(distance1-origin))+(freqmax);
  slope2 = (freqmax-freqmin)/(origin-limit);
  freq2 = (slope2*(distance1-origin))+(freqmax);
  // Print the distance readings to the serial monitor
  Serial.print("Distance from sensor 1: ");
  Serial.print(distance1);
  Serial.println(" cm");
  Serial.print("Distance from sensor 2: ");
  Serial.print(distance2);
  Serial.println(" cm");
  delay(10);
  // Als beide sensoren aan de voorwaarden voldoen, trillen beide motoren
  if (distance1 > origin && distance1 < limit && distance2 > origin && distance2 < limit)
  {
    TCA9548A(ports[0]);
    vibrate(freq1, 0.7, 0.33, 30);
    TCA9548A(ports[1]);
    vibrate(freq1, 0.7, 0.33, 30);
  }
  // Als sensor 1 aan de voorwaarden voldoet, trilt motor 1
  else if (distance1 > origin && distance1 < limit)
  {
    TCA9548A(ports[0]);
    vibrate(freq1,0.7,0.33,30);
  } 
  // Als sensor 2 aan de voorwaarden voldoet, trilt motor 2
  else if (distance2 > origin && distance2 < limit)
  {
    TCA9548A(ports[1]);
    vibrate(freq2,0.7,0.33,30);
  }

  
}
// Basic functions:

/*
1. void pulse(double intensity, double milliseconds)
pulse drives the hammer towards the closed end of the TacHammer
intensity defines the strength of the pulse. It ranges from [0-1] with 1 being strongest
When the hammer rebounds off of the repelling magnetic array, the inaudible pulse haptic
sensation is created milliseconds is the length of time the coil is activated in ms pulse is
intended to be sequenced with subsequent pulse and hit commands and if called on its own,
the hammer may travel after the rebound and strike the open end 
*/

void pulse(double intensity, double milliseconds) 
{
  int minimumint = 140;
  int maximumint = 255;
  int pwmintensity = (intensity * (maximumint - minimumint)) + minimumint;
  standbyOffB();
  PWM13 = pwmintensity;
  usdelay(milliseconds);
  standbyOnB();
}

/*
2. void singlePulse(double intensity, double millisecodns)
singlePulse includes a command to pulse the TecHammer followed by a command to brake the
hammer.
It is intended to be called on its own and should be followed by a pause command of at 
least 50 ms before any other command is called
*/
void singlePulse(double intensity, double milliseconds) 
{
  pulse(intensity, milliseconds);
  pause(3);
  pulse(intensity * 3 / 100, milliseconds * 2);
}

/*
3. void hit (double intensity, double milliseconds)
hit drives the hammer towards the open end of the TecHammer
Intensity defines the strength of the hit. It ranges from 0 to 1, with 1 being stronges
milliseconds is the lenfth of time the coil is activated in ms
When the TecHammer strikes de device housing, the audible click haptic sensation is created
*/
void hit(double intensity, double milliseconds)
 {
  int minimumint = 0;
  int maximumint = 110;
  int pwmintensity = maximumint - (intensity * (maximumint - minimumint));
  standbyOffB();
  PWM13 = pwmintensity;
  usdelay(milliseconds);
  standbyOnB();
}

/*
4. void pause(double milliseconds)
pause waits a duration of time defined in ms
*/
void pause(double milliseconds) 
{
  double us = milliseconds - ((int)milliseconds);
  standbyOnB();
  for (int i = 0; i <= milliseconds; i++) 
  {
    delay(1);
  }
  delayMicroseconds(us * 1000);
}

/*
5. void usdelay(double time)
usdelay implements a delay of the whole program in ms, it does not the same as pause
Can you describe the difference between the functions?
*/
void usdelay(double time) 
{
  double us = time - ((int)time);
  for (int i = 0; i <= time; i++) 
  {
    delay(1);
  }
  delayMicroseconds(us * 1000);
}



/*
6. void vibrate (double frequency, doub;le intensity, double duration, int dutycyle)
Vibrate repeatedly calls the pulse command to druve the TecHammer into the closed end of
the Techammer.
frequency defines the frequency of the vibration
intensity defines the strength of the vibrate. In ranges from 0 to 1, with 1 being the strongest.
duration defines the length of the vibration in s
dutycycle defines what percent of the perdio the TecHammer is active, it ranges from 0 to 100
Suggested duty cycles for vibrate
FREQ| 10| 30| 50| 70| 90|110|130|150|170|190|210|230|250|270|290
DUTY| 32| 28| 33| 36| 38| 37| 41| 47| 45| 47| 39| 40| 44| 43| 40
*/
void vibrate(double frequency, double intensity, double duration, int dutycycle) 
{
  int max_hit = 12;
  int min_hit = 1;
  int crossover = 60;
  int hitduration = 10 * dutycycle / frequency;

  boolean hold = false;
  double delayy;
  delayy = (1 / frequency * 1000) - hitduration;
  double timedown;
  timedown = duration * 1000;

  if (duration == 0)
  {
    hold =  true;
  }

  while (hold)
  {
    pulse(intensity, hitduration);
    pause(delayy);
  }

  while (timedown >= 0 && frequency < crossover)
  {
    pulse(intensity, hitduration);
    pause(3);
    pulse(0.002, delayy-3);

    timedown -= (delayy + hitduration);
  }

  while (timedown >= 0 && frequency >= crossover) 
  {
    pulse(intensity, hitduration);
    pause(delayy);
    timedown -= (delayy + hitduration);
  }
}


/*
The following are haptic effects created with the basic haptic function calls
*/

/*
void impactClick
*/
void impactClick() 
{
    Serial.println("Impact Click");
    pulse(1, 6);
    hit(1, 21);
  }


/*
void ermRumble()
Simulates the behavior of and ERM motor
*/
void ermRumble() 
{
  Serial.println("ERM Rumble");
  vibrate(30,0.7,0.33,30);
}

/*
void lraClick()
Simulates the behavior of an LRA motor
*/
void lraClick() 
{
    Serial.println("LRA Click");
    vibrate(250,1,0.01,70);
}

/*
void hearBeat()
Simulates the sensation of the "feeling" of the heart rate
*/
void heartbeat() 
{
  Serial.println("Heartbeat");
  int hrate = 0;
  for (int i=0;i<16;i++)
  {
    pulse(0.3, 45);        // "1"
    pause(12);
    pulse(0.3, 3);          // short pulse to sharpen pulse haptic
    pause(250 - hrate);

    pulse(0.1, 45);        // "2"
    pause(12);
    pulse(0.3, 3);          // short pulse to sharpen pulse haptic
    pause(510 - hrate);

    if (hrate < 200)
    {
      hrate += 15;
    }
  }   
}

/* 
void laser()
Simulates charging then firing a laser. The laser is charged while the button
is held down, then fires when let go
*/
void laser() 
{
  Serial.print("Laser Charging");

  // Charging state
  // By linking a series of increasing frequency vibrations, we create the charging feeling
  vibrate(35, 0.007, 0.07, 50);
  vibrate(40, 0.008, 0.04, 50);
  vibrate(45, 0.009, 0.04, 50);
  vibrate(60, 0.01, 0.04, 50);
  vibrate(75, 0.03, 0.04, 50);
  vibrate(80, 0.08, 0.04, 50);
  vibrate(85, 0.1, 0.04, 50);
  vibrate(88, 0.1, 0.04, 50);
  vibrate(90, 0.15, 0.04, 50);
  vibrate(130, 0.15, 0.04, 50);
  vibrate(140, 0.2, 0.04, 50);
  vibrate(160, 0.2, 0.04, 50);
  vibrate(200, 0.3, 0.05, 50);

  for (int i=0;i<4;i++)
  {
    vibrate(200, 0.5, 0.01, 50);
    pulse(.1, 2);
    vibrate(200, 0.5, 0.01, 50);
    pulse(.7, 2);
    vibrate(200, 0.5, 0.01, 50);
    pulse(.95, 2);
    vibrate(200, 0.5, 0.01, 50);
    pulse(.7, 2);
    vibrate(200, 0.5, 0.01, 50);
    pulse(.1, 2);
  }
}

void laserRelease() 
{
  Serial.println("Laser Firing");
  // Firing
  // TBy linking a series of increasing frequency vibrations, we create the discharging feeling
  hit(0.5,3);
  pulse(1, 10);
  pause(2);
  pulse(0.002,5);
  vibrate(160, 0.8, 0.02, 50);
  vibrate(150, 0.5, 0.02, 50);
  vibrate(130, 0.3, 0.03, 50);
  vibrate(120, 0.1, 0.05, 50); 
}

/*
void rifle(bool audible)
Simulates the firing of an automatic rifle.
The audible logic variable is used to change the behavior of the function. By default this
parameter is set to true in the code, change it to feel the difference
*/
bool auidibleR = true;
void rifle(bool audibleR) 
{
  if (audibleR)
  {
    Serial.println("Loud Rifle");
    // a short pulse is sent before the hit to increase the stroke of the hammer
    // so that when the hit is activated the hammer provides a stronger haptic
    pause(20);
    pulse(1, 6);
    hit(1, 21);
  }

  else 
  {
    Serial.println("Quiet Rifle");
    // a short hit is sent before the pulse to increase the stroke of the hammer
    // so that when the pulse is activated the hammer provides a stronger haptic
    hit(1,3);
    pulse(1,21);


    // a short pulse is sent after a strong pulse to catch the hammer as it rebounds
    // to ensure it does not contact the end of the controller to provde an inaudible
    // haptic      
    pause(2);
    pulse(0.002,21);
  }
}

/*
void shotgun (bool audible, bool reload)
Simulates firing a pump shotgun. Two effects can be felt:
1. Shotgun firing
2. Pumping in the next round
*/
bool audibleS = true;
bool reloadS = true;
void shotgun(bool audibleS, bool reloadS) 
{
  if (!reloadS)
  {
    if (audibleS) 
    {
        Serial.print("Loud Shotgun Firing");

        //shotgun firing
        pulse(1, 6);
        hit(1, 21);
        pause(250);
    }

    else 
    {
      Serial.print("Quiet Shotgun Firing");

      //shotgun firing
      hit(1, 3);
      singlePulse(1, 21);
      pause(200);
    }
  }

  else{
    if (audibleS) 
    {
      Serial.println("Loud Shotgun Reloading");

      //shotgun reloading
      pulse(.5, 30);
      hit(.35, 10);
      pause(180);
      hit(.37, 17);      
    }

    else 
    {
      Serial.println("Quiet Shotgun Reloading");

      //shotgun reloading
      pulse(.3, 30);
      pulse(.55, 30);
      pause(2.8);
      pulse(0.03, 60);
      pause(115);
      pulse(.55, 30);
      pause(3);
      pulse(0.03, 60);      
    }
  }
}

/*
From here, there are some test functions to check if your motor is working correctly
*/

/*
void vibrateRamp()
Plays a series of vibrates at varying frequencies and duty cycles
*/
void vibrateRamp() 
{
  double i = 0;
  for (int j=0;j<20;j++)
  {
    Serial.print("Vibrate Frequency: ");
    Serial.println(250-i);

    vibrate(200 - i, 0.7, 0.2, 70);

    if (i < 210) 
    {
      i += 10;
    }
  }  
}

/*
void pulseRamp()
Plays a series of pulses from 0 to 1 intensity levels
*/
void pulseRamp() 
{
  double i = 0;
    for (int j=0;j<20;j++)
    {
    Serial.print("Pulse Intensity: ");
    Serial.println(i);

    singlePulse(i,8);
    pause(84);
    if (i < 0.99) 
    {
      i += 0.05;
    }
  }
}

/*
void hitRamp()
Plays a series of hits from 0 to 1 intensity levels
*/
void hitRamp() 
{
  double i = 0;
    for (int j=0;j<20;j++)
    {
    Serial.print("Hit Intensity: ");
    Serial.println(i);

    pulse(i, 6);
    hit(i, 21);
    pause(100);
    if (i < 0.99) 
    {
      i += 0.05;
    }
  }
}

/*
A basic cyclic function to test different effects 
*/
void effectCycle(int count)
{
  Serial.println(count);

  if (count == 0)
  {
    for (int j=0;j<4;j++)
    {
      lraClick();
      pause(500);
    }
    pause(1000);
  }

  else if (count == 1)
  {
    for (int j=0;j<4;j++)
    {
      ermRumble();
    }
    pause(1000);
  }

  else if (count == 2)
  {
    vibrateRamp();
    pause(1000);
  }

  else if (count == 3)
  {
    for (int j=0;j<4;j++)
    {
      impactClick();
      pause(500);
    }
    pause(1000);
  }

  else if (count == 4)
  {
    for (int j=0;j<20;j++)
    {
      rifle(1);
    }
    pause(1000);
  }

  else if (count == 5)
  {
    for (int j=0;j<20;j++)
    {
      rifle(0);
    }
    pause(1000);
  }
  else if (count == 6)
  {
    for (int j=0;j<4;j++)
    {
      shotgun(1,0);
      shotgun(1,1);
      pause(500);
    }
    pause(1000);
  }   

  else if (count == 7)
  {
    for (int j=0;j<4;j++)
    {
      shotgun(0,0);
      shotgun(0,1);
      pause(500);
    }
    pause(1000);
  } 

  else if (count == 8)
  {
    laser();
    laserRelease();
    pause(1000);
  } 

  else if (count == 9)
  {
    heartbeat();
    pause(1000);
  } 

  else if (count == 10)
  {
    hitRamp();
    pause(1000);
  } 

  else if (count == 11)
  {
    pulseRamp();
    pause(1000);
  } 

  else if (count >= 11) 
  {
    //reset counters
    count = 0;
  }
}

void standbyOnB() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);            // sets register pointer to the mode register (0x01)
  Wire.write(0x43);               // Puts the device pwm mode
  Wire.endTransmission();
}

void standbyOffB() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);               // sets register pointer to the mode register (0x01)
  Wire.write(0x03);               // Sets Waveform Mode to pwm
  Wire.endTransmission();
}

void initializeDRV2605() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);            // sets register pointer to the mode register (0x01)
  Wire.write(0x00);               // clear standby
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x1D);              // sets register pointer to the Libarary Selection register (0x1D)
  Wire.write(0xA8);              // set RTP unsigned
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x03);
  Wire.write(0x02);              // set to Library B, most aggresive
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x17);               // sets full scale reference
  Wire.write(0xff);               //
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);               // sets register pointer to the mode register (0x01)
  Wire.write(0x03);               // Sets Mode to pwm
  Wire.endTransmission();

  delay(100);

}

void TCA9548A(uint8_t bus){
  Wire.beginTransmission(TCA);
  Wire.write(1 << bus);
  Wire.endTransmission();
}

/*
DEAR STUDENTS, PLEASE DO NOT MODIFY THIS SECTION
iF YOU NEED ANY ASSISTANCE, PLEASE NOTIFY THE TA OR THE MAIN LECTURER
*/

// Configure the PWM clock
#define PWM12k  5   //  11719 Hz

void pwm13configure() {
  // TCCR4A configuration
  TCCR4A = 0;

  // TCCR4B configuration
  TCCR4B = PWM12k;

  // TCCR4C configuration
  TCCR4C = 0;

  // TCCR4D configuration
  TCCR4D = 0;

  // TCCR4D configuration
  TCCR4D = 0;

  // PLL Configuration
  // Use 96MHz / 2 = 48MHz
  PLLFRQ = (PLLFRQ & 0xCF) | 0x30;
  // PLLFRQ=(PLLFRQ&0xCF)|0x10; // Will double all frequencies

  // Terminal count for Timer 4 PWM
  OCR4C = 255;

  //pwmSet6();
  pwmSet13();
}


// Set PWM to D13 (Timer4 A)
void pwmSet13() {
  OCR4A = 0;  
  DDRC |= _BV(7); 
  TCCR4A = 0x82; 
}

